@@ -1,6 +1,21 @@
 ﻿import json
 
 
+class JSONRPCProtocol(object):
+
+    """ JSON-RPC protocol implementation."""
+
+    JSONRPC_VERSION = "2.0"
+
+    @classmethod
+    def create_request(cls, method, params, _id=None):
+        return JSONRPCRequest(method, params, _id=_id)
+
+    @classmethod
+    def parse_request(cls, json_str):
+        return JSONRPCRequest.from_json(json_str)
+
+
 class JSONRPCRequest(object):
 
     """ A rpc call is represented by sending a Request object to a Server.
@@ -41,18 +56,15 @@ class JSONRPCRequest(object):
     deserialize = staticmethod(json.loads)
 
     def __init__(self, method=None, params=None, _id=None):
-        if not isinstance(method, str):
-            raise ValueError("method should be string")
-
         assert isinstance(params, (list, dict))
         assert _id is None or isinstance(_id, (int, str))
 
-        self.method = method
+        self.method = str(method)
         self.params = params
         self.id = _id
 
     @property
-    def json(self):
+    def dict(self):
         data = dict(
             jsonrpc=JSONRPCProtocol.JSONRPC_VERSION,
             method=self.method,
@@ -62,12 +74,29 @@ class JSONRPCRequest(object):
         if self.id:
             data["id"] = self.id
 
-        return self.serialize(data)
+        return data
+
+    @property
+    def args(self):
+        return tuple(self.params) if isinstance(self.params, list) else ()
+
+    @property
+    def kwargs(self):
+        return self.params if isinstance(self.params, dict) else {}
+
+    @property
+    def json(self):
+        return self.serialize(self.dict)
 
     @classmethod
     def from_json(cls, json_str):
         data = cls.deserialize(json_str)
-        return JSONRPCRequest(method=data["method"], params=data["params"])
+        if not isinstance(data, list):
+            return JSONRPCRequest(method=data["method"], params=data["params"])
+        else:
+            return [
+                JSONRPCRequest(method=d["method"], params=d["params"])
+                for d in data]
 
     def respond_error(self, error):
         data = JSONRPCResponse(error=error, _id=self.id)._dict
@@ -84,7 +113,16 @@ class JSONRPCRequest(object):
         :return bool:
 
         """
-        return self._id is None
+        return self.id is None
+
+
+class JSONRPCBatchRequest(object):
+    def __init__(self, *requests):
+        self.requests = requests
+
+    @property
+    def json(self):
+        return json.dumps([r.dict for r in self.requests])
 
 
 class JSONRPCResponse(object):
@@ -117,6 +155,8 @@ class JSONRPCResponse(object):
 
     """
 
+    serialize = staticmethod(json.dumps)
+
     def __init__(self, result=None, error=None, _id=None):
         self.result = result
         self.error = error
@@ -131,17 +171,6 @@ class JSONRPCResponse(object):
 
         return data
 
-
-class JSONRPCProtocol(object):
-
-    """ JSON-RPC protocol implementation."""
-
-    JSONRPC_VERSION = "2.0"
-
-    @classmethod
-    def create_request(cls, method, params, _id=None):
-        return JSONRPCRequest(method, params, _id=_id)
-
-    @classmethod
-    def parse_request(cls, request):
-        return JSONRPCRequest.from_json(request)
+    @property
+    def json(self):
+        return self.serialize(self._dict)
